#include "mainwindow.h"
#include "ui_mainwindow.h"
//#include <QtGui>
//#include <QtOpenGL>
//#include <QGLWidget>
//#include "fmod.h"

#include"settings.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{

    ui->setupUi(this);
    channel = 0;
    status_label = new QLabel("Starting...");
    ui->statusBar->addWidget(status_label);

    Settings* settings = new Settings();
    settings->show();


	//init global params
    pi = 3.14159265;

    spectrum_size = 4096;


	//BASS SETTINGS:
    bass_circle_radius = 70.0;
    bass_segments_radius_min = 8.0;
    bass_segments_radius_amplifier = 16.0;
    bass_segments = 100;
 
    kick_segment = 17;
    kick_circle_radius = 15.0;
	
	bass_amplifier = 10.0;
	kick_amplifier = 5.0;

	bass_treshold = 0.0001;
	//kick_treshold = 0.095
	kick_treshold = 0.06;

	//GUITAR SETTINGS:
    guitar_circle_radius = 65.0;
    guitar_segments_radius_min = 4.0;
	guitar_segments_radius_amplifier = 55.0;
	guitar_segments = 160;

    guitar_amplifier = -20.0;

	guitar_treshold;	

    guitar_segments = 160;


	//init GUI components
    ui->graphicsView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    spectrum_scene = new QGraphicsScene();
    spectrum_scene->setBackgroundBrush(QBrush(QColor(255,255,255,255)));
    ui->graphicsView->setScene(spectrum_scene);
    ui->graphicsView->setRenderHints(QPainter::Antialiasing);


    //bass = spectrum_scene->addEllipse(0,0,radius*2,radius*2,QPen(QColor(255,255,255,100)),QBrush(QColor(0,0,0,0)));

    text_time = spectrum_scene->addText("-",QFont("Arial",8,-1,false));
    text_time->setDefaultTextColor(QColor(255,255,255,255));

    time.start();

    time1 = time.elapsed();

    spectrumarray = new float[spectrum_size];


    for(int i = 0; i < guitar_segments; i++){
        QGraphicsEllipseItem* ellipse;
        ellipse = spectrum_scene->addEllipse(0,0,4,4,QPen(QColor(0,255,0,255)),QBrush(QColor(0,0,255,255)));

        //pre calc stuff...
        float calc = ((float)(i))/((float)(bass_segments))*360.0*(pi/180.0);
        ellipse->setPos(sin(calc)*65.0 -2 , cos(calc)*65.0 -2);
        ellipse_list2.append(ellipse);
    }

    for(int i = 0; i < bass_segments; i++){
        QGraphicsEllipseItem* ellipse;
        if(i == kick_segment){
            ellipse = spectrum_scene->addEllipse(0,0,4,4,QPen(QColor(255,0,0,255)),QBrush(QColor(0,0,0,0)));
        }
        else{
            ellipse = spectrum_scene->addEllipse(0,0,4,4,QPen(QColor(0,0,255,255)),QBrush(QColor(0,255,0,255)));
        }
        //pre calc stuff...
        float calc = ((float)(i))/((float)(bass_segments))*360.0*(pi/180.0);
        ellipse->setPos(sin(calc)*60.0 -2 , cos(calc)*60.0 -2);
        ellipse_list1.append(ellipse);
    }



    connect(settings,SIGNAL(set_bass_amplifier_signal(qreal)),this,SLOT(set_bass_amplifier(qreal)));
    connect(settings,SIGNAL(set_bass_circle_radius_signal(qreal)),this,SLOT(set_bass_circle_radius(qreal)));
    connect(settings,SIGNAL(set_bass_segments_radius_min_signal(qreal)),this,SLOT(set_bass_segments_radius_min(qreal)));
    connect(settings,SIGNAL(set_bass_segments_radius_amplifier_signal(qreal)),this,SLOT(set_bass_segments_radius_amplifier(qreal)));
    connect(settings,SIGNAL(set_bass_treshold_signal(qreal)),this,SLOT(set_bass_treshold(qreal)));

    connect(settings,SIGNAL(set_guitar_amplifier_signal(qreal)),this,SLOT(set_guitar_amplifier(qreal)));
    connect(settings,SIGNAL(set_guitar_circle_radius_signal(qreal)),this,SLOT(set_guitar_circle_radius(qreal)));
    connect(settings,SIGNAL(set_guitar_segments_radius_min_signal(qreal)),this,SLOT(set_guitar_segments_radius_min(qreal)));
    connect(settings,SIGNAL(set_guitar_segments_radius_amplifier_signal(qreal)),this,SLOT(set_guitar_segments_radius_amplifier(qreal)));
    connect(settings,SIGNAL(set_guitar_treshold_signal(qreal)),this,SLOT(set_guitar_treshold(qreal)));

    connect(settings,SIGNAL(set_kick_amplifier_signal(qreal)),this,SLOT(set_kick_amplifier(qreal)));
    connect(settings,SIGNAL(set_kick_segment_signal(int)),this,SLOT(set_kick_segment(int)));
    connect(settings,SIGNAL(set_kick_treshold_signal(qreal)),this,SLOT(set_kick_treshold(qreal)));


    connect(ui->pushButton_play,SIGNAL(released()),this,SLOT(play_song()));
    //connect(settings,SIGNAL(released()),this,SLOT(play_song()));

}

void MainWindow::set_bass_circle_radius(qreal circle_radius){
    this->bass_circle_radius = circle_radius;
}

void MainWindow::set_bass_segments_radius_min(qreal segments_radius_min){
    this->bass_segments_radius_min = segments_radius_min;
}

void MainWindow::set_bass_segments_radius_amplifier(qreal segments_radius_amplifier){
    this->bass_segments_radius_amplifier = segments_radius_amplifier;
}

void MainWindow::set_kick_segment(int segment){
    this->kick_segment = segment;
}

void MainWindow::set_kick_circle_radius(qreal circle_radius){
    this->kick_circle_radius = circle_radius;
}

void MainWindow::set_bass_amplifier(qreal amplifier){
    this->bass_amplifier = amplifier;
}

void MainWindow::set_kick_amplifier(qreal amplifier){
    this->kick_amplifier = amplifier;
}

void MainWindow::set_bass_treshold(qreal treshold){
    this->bass_treshold = treshold;
}

void MainWindow::set_kick_treshold(qreal treshold){
    this->kick_treshold = treshold;
}



void MainWindow::set_guitar_circle_radius(qreal circle_radius){
    this->guitar_circle_radius = circle_radius;
}

void MainWindow::set_guitar_segments_radius_min(qreal segments_radius_min){
    this->guitar_segments_radius_min = segments_radius_min;
}

void MainWindow::set_guitar_segments_radius_amplifier(qreal segments_radius_amplifier){
    this->guitar_segments_radius_amplifier = segments_radius_amplifier;
}

void MainWindow::set_guitar_amplifier(qreal amplifier){
    this->guitar_amplifier = amplifier;
}

void MainWindow::set_guitar_treshold(qreal treshold){
    this->guitar_treshold = treshold;
}


void MainWindow::play_song(){
    timer = new QTimer();
    //timer->setInterval(10);
    timer->setInterval(1);
    connect(timer,SIGNAL(timeout()),this,SLOT(draw_animation()));

    open_snd("D:/spectr/spectr/beast.mp3");
}


void MainWindow::update_status_bar(QString text){
    status_label->setText(text);
}

float MainWindow::min(float a, float b){
    if(a < b){
        return a;
    }
    return b;
}

void MainWindow::draw_animation(){
    //FMOD_DSP_FFT_WINDOW_BLACKMANHARRIS , FMOD_DSP_FFT_WINDOW_HAMMING, FMOD_DSP_FFT_WINDOW_RECT
    FMOD_Channel_GetSpectrum( channel, spectrumarray, spectrum_size, 0, FMOD_DSP_FFT_WINDOW_BLACKMANHARRIS);


/*
    //NORMAILZE PASS

    //get max
    float max = 0.0;
    for(int i = 0; i < 128; i++){
        if(spectrumarray[i] > max){
            max = spectrumarray[i];
        }
    }

    //normalize
    for(int i = 0; i < 128; i++){
        spectrumarray[i] = spectrumarray[i] / max;
    }
*/


    QSize size = ui->graphicsView->size();
    int size_y = size.height();
    int size_x = size.width();




    //BASS CIRCLES
    float max1 = 0;
    for(int i = 0; i < bass_segments; i++){
        if(spectrumarray[i]>max1){
            max1 = spectrumarray[i];
        }
    }

    //bass->setPos(-radius,-radius);
    //bass->setRect(-bass_circle_radius*(1.0+spectrumarray[bass_segment]*7.0),-bass_circle_radius*(1.0+spectrumarray[bass_segment]*7.0),
    //              bass_circle_radius*2*(1.0+spectrumarray[bass_segment]*7.0),bass_circle_radius*2*(1.0+spectrumarray[bass_segment]*7.0));

    //bass->setPen(QColor(255*4*((spectrumarray[bass_segment])/1.0),255-255*4*((spectrumarray[bass_segment])/1.0),255-255*4*((spectrumarray[bass_segment])/1.0),255));
    //bass->setPen(QColor(0,0,0,0));

    for(int i = 0; i < ellipse_list1.length(); i++){
        //pre calc stuff...
		//angle of segment i in radiants
        float calc = ((float)(i))/((float)(ellipse_list1.length()-1))*360.0*(pi/180.0);
	
		//amplitude of segment
        float calc2;
        if(spectrumarray[kick_segment]>kick_treshold){		//BLACKMANHARRIS
            calc2 = bass_circle_radius*(1.0+spectrumarray[i]*bass_amplifier)*(1.0+spectrumarray[kick_segment]*kick_amplifier);
            //ellipse_list1.at(i)->setPen(QColor(0,0,255,255));
            ellipse_list1.at(i)->setRect(0,0,kick_circle_radius*2.0,kick_circle_radius*2.0);
            ellipse_list1.at(i)->setPos(sin(calc)*calc2 - kick_circle_radius,
                                        cos(calc)*calc2 - kick_circle_radius);
        }
        else{
            calc2 = bass_circle_radius*(1.0+spectrumarray[i]*bass_amplifier);
            //ellipse_list1.at(i)->setPen(QColor(0,0,255,185));
            ellipse_list1.at(i)->setRect(0,0,bass_segments_radius_min*(1.0+spectrumarray[i]*bass_segments_radius_amplifier)*2.0,
                                             bass_segments_radius_min*(1.0+spectrumarray[i]*bass_segments_radius_amplifier)*2.0);
            ellipse_list1.at(i)->setPos(sin(calc)*calc2 -bass_segments_radius_min*(1.0+spectrumarray[i]*bass_segments_radius_amplifier),
                                        cos(calc)*calc2 -bass_segments_radius_min*(1.0+spectrumarray[i]*bass_segments_radius_amplifier));
        }

        //ellipse_list1.at(i)->setPen(QColor(255*2*((spectrumarray[i])/1.0),0,255,255));
    }


    //SECOND RING OF RINGS
    float max2 = 0;
    for(int i = bass_segments+60; i < bass_segments + guitar_segments+70; i++){
        if(spectrumarray[i]>max2){
            max2 = spectrumarray[i];
        }
    }

    for(int i = 0; i < ellipse_list2.length(); i++){
        //pre calc stuff...
        float calc = ((float)(i))/((float)(ellipse_list2.length()-1))*360.0*(pi/180.0);
        //float calc2 = 10.0*(1.0+spectrumarray[guitar_segments+70 + i]*125.0);				//Rect
        float calc2 = guitar_circle_radius*(1.0+spectrumarray[guitar_segments+70+i]*guitar_amplifier);				//blackmanharris
        ellipse_list2.at(i)->setRect(0,0,guitar_segments_radius_min*2.0,
                                         guitar_segments_radius_min*2.0);
        ellipse_list2.at(i)->setPos(sin(calc)*calc2-guitar_segments_radius_min,
                                    cos(calc)*calc2-guitar_segments_radius_min);
        //ellipse_list2.at(i)->setPen(QColor(255*4*((spectrumarray[guitar_segments+70+i])/1.0),255,0,255));
    }


    time2 = time.elapsed();
    text_time->setPos(50,50);
    text_time->setPlainText(QString::number(time2-time1));
    time1 = time2;

}

void MainWindow::init_animation(){
    timer->start();
}

void MainWindow::open_snd(QString snd_path){

    //statusBar()->removeWidget(status_label);

    result = FMOD_System_Create(&system);
    if(result != FMOD_OK){
        qDebug("FMOD error 1! (%d) %s\n", result, FMOD_ErrorString(result));
    }

    result =FMOD_System_Init(system, 32 ,FMOD_INIT_NORMAL,0 );
    if(result != FMOD_OK){
        qDebug("FMOD error 2! (%d) %s\n", result, FMOD_ErrorString(result));
    }


    result = FMOD_System_CreateStream(system, snd_path.toUtf8() ,FMOD_SOFTWARE,0,&snd1 );
    if(result != FMOD_OK){
        qDebug("FMOD error 3! (%d) %s\n", result, FMOD_ErrorString(result));
    }


    result = FMOD_System_PlaySound(system,FMOD_CHANNEL_FREE, snd1, false, &channel );
    if(result != FMOD_OK){
        qDebug("FMOD error 4! (%d) %s\n", result, FMOD_ErrorString(result));
    }

    update_status_bar("<font color=\"green\">Playing</font>: " + snd_path.toUtf8());
    ui->statusBar->addWidget(status_label);
    init_animation();
}

MainWindow::~MainWindow()
{
    delete ui;
}
