#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "fmod.h"
#include "fmod_dsp.h"
#include "fmod_errors.h"

#include <math.h>

#include <QList>
#include <QLabel>
#include <QGraphicsScene>
#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>
#include <QRectF>
#include <QTimer>
#include <QTime>

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    void open_snd(QString snd_path);
    void update_status_bar(QString text);

    float min(float a, float b);

    void init_animation();

	//////////////////////////////////////////////////////////////////
	//	GLOBALS
	//////////////////////////////////////////////////////////////////

	//FMOD vars
    FMOD_SYSTEM *system;
    FMOD_SOUND *snd1;
    FMOD_CHANNEL *channel;
    FMOD_RESULT result;

    QLabel* status_label;

	//Frequencies
    int spectrum_size;
	

	

	//bass circle params
    qreal bass_circle_radius;			//radius for circle
	qreal bass_segments_radius_min;	//initial/standby radius for segments
	qreal bass_segments_radius_max;	//maximum radius for segments
    int bass_segments;				//segments count
 
    int kick_segment;				//circle/frequency for kick
	qreal kick_circle_radius;			//kick circle radius
	
	qreal bass_amplifier;
	qreal kick_amplifier;

	qreal bass_treshold;
	qreal kick_treshold;
	

	//guitar circle params
	qreal guitar_circle_radius;		//radius for circle
    qreal guitar_segments_radius_min;	//initial/standby radius for segments
	qreal guitar_segments_radius_max;	//maximum radius for segments
	int guitar_segments;			//segments count

	qreal guitar_amplifier;

	qreal guitar_treshold;	


	//PI for circular arrangement of circles
    float pi;

    QGraphicsTextItem* text_time;
    QTime time;
    int time1;
    int time2;

    QList<QGraphicsEllipseItem*> ellipse_list1;
    QList<QGraphicsEllipseItem*> ellipse_list2;

    QList<QGraphicsRectItem*> rect_list1;
    QList<QGraphicsRectItem*> rect_list2;
    QList<QGraphicsRectItem*> rect_list3;

    QList<QGraphicsLineItem*> line_list1;
    QList<QGraphicsLineItem*> line_list2;

    QGraphicsEllipseItem* bass;
    QGraphicsRectItem* bass2;
    QGraphicsScene* spectrum_scene;
    QPainter* painter;

    float* spectrumarray;

    QTimer* timer;

	//////////////////////////////////////////////////////////////////
	//	SLOTS
	//////////////////////////////////////////////////////////////////

public slots:
    void draw_animation();
    
	//////////////////////////////////////////////////////////////////
	//	PRIVATES
	//////////////////////////////////////////////////////////////////

private:
    Ui::MainWindow *ui;
};

#endif // MAINWINDOW_H
